/*** Procedure to run tests of predictability ***/

proc(3) = predtesttrendU(ret,pred,maxp,ii);

local zcrit, p, n, T, r, x, Dx, Lx, Z, i, 
    coef, v, Sv2, varmat, SErho, e, psi, df, 
    rhogls, xbar, inter, mu, Zbar, resid, dfgls, 
    conf, u, beta, Sigma, Kurt, delta, norma, omega2, 
    normbeta, SEbeta, tstat, tbeta, cL, cU, 
    critL, critR, Qbeta, rhoL, rhoU, correct, rbar, pols, pgls,ip,cdf,cdfgls,alphahat,dhat,resinit,siginit,dmpred,dfnb,tr,LBT;

zcrit = cdfni(1-0.05*unionscale[ii]);
   

/* Choose lag length p by BIC */

p = bic(pred,maxp);

/* Sample size */

n = rows(ret);
tr=seqa(1,1,n);
T = n-p;

/* Create variables */

r = ret[p+1:n];
x = pred[p+1:n];
Dx = pred[p+1:n]-pred[p:n-1];

Lx = ones(T,1)~pred[p:n-1]~tr[p+1:n];

for i (1,p,1);
    if i==1;
        Z = Lx;
    else;
        Z = Z~(pred[p-i+2:n-i+1]-pred[p-i+1:n-i]);
    endif;
endfor;


/* Estimate AR(1) for predictor variable */

coef = x/Lx;
v = x-Lx*coef;

Sv2 = v'v/(T-3);

varmat = Sv2*invpd(Lx'Lx);
SErho = sqrt(varmat[2,2]);

/* Estimate AR(p) for predictor variable */

coef = Dx/Z;
e = Dx-Z*coef;

if p==1;
    psi = 0;
else;
    psi = coef[4:p+2];
endif;

/* Compute DF t-statistic */

varmat = (e'e)/(T-p-2)*invpd(Z'Z);

df = coef[2]/sqrt(varmat[2,2]);

zbar=GLS_t(pred);
dfgls=DFTEST(zbar,p-1);

/* Compute 95% confidence interval for c and rho */

conf = zeros(1,4);

conf[3] = Lconf(dfgls,DFGLSbelttrend,5);
conf[4] = Uconf(dfgls,DFGLSbelttrend,5);

conf[1] = 1+conf[3]/T;
conf[2] = 1+conf[4]/T;


/* Predictive regression */

coef = r/Lx;
u = r-Lx*coef;

beta = coef[2];

/* Estimate covariance matrix */

Sigma = (u~e)'(u~e)/(T-3);

delta = Sigma[1,2]/sqrt(Sigma[1,1]*Sigma[2,2]);
norma = sqrt(Sigma[2,2]/Sigma[1,1]);

omega2 = Sigma[2,2]/(1-sumc(psi))^2;

/* Estimate excess kurtosis */

Kurt = meanc((u~e)^4)./(meanc((u~e)^2)^2)-3*ones(2,1);

/* Compute point estimate and t-stat for beta */

normbeta = norma*beta;

varmat = Sigma[1,1]*invpd(Lx'Lx);
SEbeta = sqrt(varmat[2,2]);

tstat = beta/SEbeta;
LBT= norma*(beta-1.645*SEbeta);

/* Compute 90% confidence interval for beta using t-stat */

tbeta = zeros(1,4);

cL = BonLconf(df,DFbelttrend,-abs(delta),tconftr[.,1]);
cU = BonUconf(df,DFbelttrend,-abs(delta),tconftr[.,1]);
critL = BonLcrit(cL,cU,-abs(delta),tquantltrend);


cL = BonLconf(df,DFbelttrend,-abs(delta),ceil(tconftr[.,2]));
cU = BonUconf(df,DFbelttrend,-abs(delta),ceil(tconftr[.,2]));
critR = BonRcrit(cL,cU,-abs(delta),tquantrtrend);


if delta<=0;
    tbeta[1] = norma*(beta-critR*SEbeta);
    tbeta[2] = norma*(beta-critL*SEbeta);
else;
    tbeta[1] = norma*(beta+critL*SEbeta);
    tbeta[2] = norma*(beta+critR*SEbeta);
endif;

cL = BonLconf(dfgls,DFGLSbelttrend,-abs(delta),tconftrgls[.,1]);
cU = BonUconf(dfgls,DFGLSbelttrend,-abs(delta),tconftrgls[.,1]);
critL = BonLcrit(cL,cU,-abs(delta),tquantltrend);


cL = BonLconf(dfgls,DFGLSbelttrend,-abs(delta),tconftrgls[.,2]);
cU = BonUconf(dfgls,DFGLSbelttrend,-abs(delta),tconftrgls[.,2]);
critR = BonRcrit(cL,cU,-abs(delta),tquantrtrend);


if delta<=0;
    tbeta[3] = norma*(beta-critR*SEbeta);
    tbeta[4] = norma*(beta-critL*SEbeta);
else;
    tbeta[3] = norma*(beta+critL*SEbeta);
    tbeta[4] = norma*(beta+critR*SEbeta);
endif;

Qbeta = zeros(1,13);

/* Compute 90% confidence interval for beta using Q-stat and DF-GLS */

cL = BonLconf(dfgls,DFGLSbelttrend,-abs(delta),Qconftr[.,1]);
cU = BonUconf(dfgls,DFGLSbelttrend,-abs(delta),ceil(Qconftr[.,2].*unionscale));


rhoL = 1+cL/T;
rhoU = 1+cU/T;

correct = (T-2)/2*Sigma[1,2]/sqrt(Sigma[2,2]*omega2)*(omega2/Sv2-1)*SErho^2;

/* At LOWER confidence interval for rho */

rbar = r-Sigma[1,2]/sqrt(Sigma[2,2]*omega2)*(x-rhoL*Lx[.,2]);

coef = rbar/Lx;

Qbeta[1] = norma*(coef[2]+correct-zcrit*sqrt(1-delta^2)*SEbeta);
Qbeta[2] = norma*(coef[2]+correct+zcrit*sqrt(1-delta^2)*SEbeta);

/* At UPPER confidence interval for rho */

rbar = r-Sigma[1,2]/sqrt(Sigma[2,2]*omega2)*(x-rhoU*Lx[.,2]);

coef = rbar/Lx;

Qbeta[3] = norma*(coef[2]+correct-zcrit*sqrt(1-delta^2)*SEbeta);
Qbeta[4] = norma*(coef[2]+correct+zcrit*sqrt(1-delta^2)*SEbeta);

/* Bonferroni confidence interval */

if delta<=0;
    Qbeta[5] = Qbeta[3];
    Qbeta[6] = Qbeta[2];
else;
    Qbeta[5] = Qbeta[1];
    Qbeta[6] = Qbeta[4];
endif;


/* Compute 90% confidence interval for beta using Q-stat and DF-OLS */

cL = BonLconf(df,DFbelttrend,-abs(delta),Qconftrols[.,1]);
cU = BonUconf(df,DFbelttrend,-abs(delta),Qconftrols[.,2]);

rhoL = 1+cL/T;
rhoU = 1+cU/T;

correct = (T-2)/2*Sigma[1,2]/sqrt(Sigma[2,2]*omega2)*(omega2/Sv2-1)*SErho^2;

/* At LOWER confidence interval for rho */

rbar = r-Sigma[1,2]/sqrt(Sigma[2,2]*omega2)*(x-rhoL*Lx[.,2]);

coef = rbar/Lx;

Qbeta[7] = norma*(coef[2]+correct-zcrit*sqrt(1-delta^2)*SEbeta);
Qbeta[8] = norma*(coef[2]+correct+zcrit*sqrt(1-delta^2)*SEbeta);

/* At UPPER confidence interval for rho */

rbar = r-Sigma[1,2]/sqrt(Sigma[2,2]*omega2)*(x-rhoU*Lx[.,2]);

coef = rbar/Lx;

Qbeta[9] = norma*(coef[2]+correct-zcrit*sqrt(1-delta^2)*SEbeta);
Qbeta[10] = norma*(coef[2]+correct+zcrit*sqrt(1-delta^2)*SEbeta);

/* Bonferroni confidence interval */

if delta<=0;
    Qbeta[11] = Qbeta[9];
    Qbeta[12] = Qbeta[8];
else;
    Qbeta[11] = Qbeta[7];
    Qbeta[12] = Qbeta[10];
endif;

retp(p~delta~df~dfgls~conf,tstat~normbeta~tbeta~Qbeta,LBT);

endp;

proc GLS_t(y);
    local n,y_ers,ct,z,beta,yd,phi_ers_ct;
    n = rows(y);
    phi_ers_ct=1-(13.5/n);
    y_ers = y[1]|(y[2:n]-phi_ers_ct*y[1:n-1]);
	ct = ones(n,1)~seqa(1,1,n);
	z = ct[1,.]|(ct[2:n,.]-phi_ers_ct*ct[1:n-1,.]);
	beta = inv(z'z)*z'y_ers;
	yd = y-ct*beta;
    retp(yd);
endp;


proc (1) = DFTEST(r,k);
	local dr,x,dep,invxx,b,res,s2,adf,sig2,w2;
	dr = r-lag1(r);
	if k == 0;
		x = trimr(lag1(r),1,0);
	else;
		x = trimr(lag1(r)~shiftr(dr',seqa(1,1,k),0)',k+1,0);
	endif;
	dep = trimr(dr,k+1,0);
	invxx = inv(x'x);
	b = invxx*x'dep;
	res = dep-x*b;
	s2 = res'res/(rows(x)-cols(x));
	adf = b[1]/sqrt(s2*invxx[1,1]);
	sig2 = (res'res/(rows(res)-1));
	if k == 0;
		w2 = sig2;
	else;
		w2 = sig2/((1-sumc(b[2:k+1]))^2);
	endif;
    retp(adf);
endp;
